/**
 * 
 *   Copyright (c) 2004-2015, Istvan David, Istvan Rath and Daniel Varro
 *   All rights reserved. This program and the accompanying materials
 *   are made available under the terms of the Eclipse Public License v1.0
 *   which accompanies this distribution, and is available at
 *   http://www.eclipse.org/legal/epl-v10.html
 *  
 *   Contributors:
 *   Istvan David - initial API and implementation
 *  
 */
package org.eclipse.viatra.cep.core.engine.compiler.internal;

import org.eclipse.viatra.cep.core.engine.compiler.internal.ComplexPatternQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.internal.ComplexTransitionQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.AndPatternQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.AtomicEventPatternQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.ComplexAndTransitionQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.ComplexFollowsTransitionQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.ComplexNotTransitionQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.ComplexOrTransitionQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.FollowsPatternQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.NegativeTransitionQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.NonUnfoldedNotTransitionQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.NotPatternQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.OrPatternQuerySpecification;
import org.eclipse.viatra.cep.core.engine.compiler.util.TranslatedEventPatternQuerySpecification;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedPatternGroup;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;

/**
 * A pattern group formed of all patterns defined in mappingPatterns.vql.
 * 
 * <p>A private group that includes private patterns as well. Only intended use case is for pattern testing.
 * 
 * <p> From package org.eclipse.viatra.cep.core.engine.compiler, the group contains the definition of the following patterns: <ul>
 * <li>translatedEventPattern</li>
 * <li>atomicEventPattern</li>
 * <li>complexPattern</li>
 * <li>followsPattern</li>
 * <li>orPattern</li>
 * <li>andPattern</li>
 * <li>notPattern</li>
 * <li>complexTransition</li>
 * <li>negativeTransition</li>
 * <li>complexFollowsTransition</li>
 * <li>complexOrTransition</li>
 * <li>complexAndTransition</li>
 * <li>nonUnfoldedNotTransition</li>
 * <li>complexNotTransition</li>
 * </ul>
 * 
 * @see IPatternGroup
 * 
 */
@SuppressWarnings("all")
public final class MappingPatternsAll extends BaseGeneratedPatternGroup {
  /**
   * Access the pattern group.
   * 
   * @return the singleton instance of the group
   * @throws ViatraQueryException if there was an error loading the generated code of pattern specifications
   * 
   */
  public static MappingPatternsAll instance() throws ViatraQueryException {
    if (INSTANCE == null) {
        INSTANCE = new MappingPatternsAll();
    }
    return INSTANCE;
  }
  
  private static MappingPatternsAll INSTANCE;
  
  private MappingPatternsAll() throws ViatraQueryException {
    querySpecifications.add(TranslatedEventPatternQuerySpecification.instance());
    querySpecifications.add(AtomicEventPatternQuerySpecification.instance());
    querySpecifications.add(ComplexPatternQuerySpecification.instance());
    querySpecifications.add(FollowsPatternQuerySpecification.instance());
    querySpecifications.add(OrPatternQuerySpecification.instance());
    querySpecifications.add(AndPatternQuerySpecification.instance());
    querySpecifications.add(NotPatternQuerySpecification.instance());
    querySpecifications.add(ComplexTransitionQuerySpecification.instance());
    querySpecifications.add(NegativeTransitionQuerySpecification.instance());
    querySpecifications.add(ComplexFollowsTransitionQuerySpecification.instance());
    querySpecifications.add(ComplexOrTransitionQuerySpecification.instance());
    querySpecifications.add(ComplexAndTransitionQuerySpecification.instance());
    querySpecifications.add(NonUnfoldedNotTransitionQuerySpecification.instance());
    querySpecifications.add(ComplexNotTransitionQuerySpecification.instance());
  }
}
