/**
 * 
 *   Copyright (c) 2004-2015, Istvan David, Istvan Rath and Daniel Varro
 *   All rights reserved. This program and the accompanying materials
 *   are made available under the terms of the Eclipse Public License v1.0
 *   which accompanies this distribution, and is available at
 *   http://www.eclipse.org/legal/epl-v10.html
 *  
 *   Contributors:
 *   Istvan David - initial API and implementation
 *  
 */
package org.eclipse.viatra.cep.core.engine.compiler.util;

import com.google.common.collect.Sets;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.viatra.cep.core.engine.compiler.OrPatternMatch;
import org.eclipse.viatra.cep.core.engine.compiler.OrPatternMatcher;
import org.eclipse.viatra.cep.core.engine.compiler.internal.ComplexPatternQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFQuerySpecification;
import org.eclipse.viatra.query.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.psystem.PBody;
import org.eclipse.viatra.query.runtime.matchers.psystem.PVariable;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.PositivePatternCall;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameterDirection;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.QueryInitializationException;
import org.eclipse.viatra.query.runtime.matchers.tuple.FlatTuple;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuples;

/**
 * A pattern-specific query specification that can instantiate OrPatternMatcher in a type-safe way.
 * 
 * @see OrPatternMatcher
 * @see OrPatternMatch
 * 
 */
@SuppressWarnings("all")
public final class OrPatternQuerySpecification extends BaseGeneratedEMFQuerySpecification<OrPatternMatcher> {
  private OrPatternQuerySpecification() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static OrPatternQuerySpecification instance() throws ViatraQueryException {
    try{
        return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
        throw processInitializerError(err);
    }
  }
  
  @Override
  protected OrPatternMatcher instantiate(final ViatraQueryEngine engine) throws ViatraQueryException {
    return OrPatternMatcher.on(engine);
  }
  
  @Override
  public OrPatternMatcher instantiate() throws ViatraQueryException {
    return OrPatternMatcher.create();
  }
  
  @Override
  public OrPatternMatch newEmptyMatch() {
    return OrPatternMatch.newEmptyMatch();
  }
  
  @Override
  public OrPatternMatch newMatch(final Object... parameters) {
    return OrPatternMatch.newMatch((org.eclipse.viatra.cep.core.metamodels.events.ComplexEventPattern) parameters[0]);
  }
  
  /**
   * Inner class allowing the singleton instance of {@link JvmGenericType: org.eclipse.viatra.cep.core.engine.compiler.util.OrPatternQuerySpecification (visibility: PUBLIC, simpleName: OrPatternQuerySpecification, identifier: org.eclipse.viatra.cep.core.engine.compiler.util.OrPatternQuerySpecification, deprecated: <unset>) (abstract: false, static: false, final: true, packageName: org.eclipse.viatra.cep.core.engine.compiler.util) (interface: false, strictFloatingPoint: false, anonymous: false)} to be created 
   *     <b>not</b> at the class load time of the outer class, 
   *     but rather at the first call to {@link JvmGenericType: org.eclipse.viatra.cep.core.engine.compiler.util.OrPatternQuerySpecification (visibility: PUBLIC, simpleName: OrPatternQuerySpecification, identifier: org.eclipse.viatra.cep.core.engine.compiler.util.OrPatternQuerySpecification, deprecated: <unset>) (abstract: false, static: false, final: true, packageName: org.eclipse.viatra.cep.core.engine.compiler.util) (interface: false, strictFloatingPoint: false, anonymous: false)#instance()}.
   * 
   * <p> This workaround is required e.g. to support recursion.
   * 
   */
  private static class LazyHolder {
    private final static OrPatternQuerySpecification INSTANCE = new OrPatternQuerySpecification();
    
    /**
     * Statically initializes the query specification <b>after</b> the field {@link #INSTANCE} is assigned.
     * This initialization order is required to support indirect recursion.
     * 
     * <p> The static initializer is defined using a helper field to work around limitations of the code generator.
     * 
     */
    private final static Object STATIC_INITIALIZER = ensureInitialized();
    
    public static Object ensureInitialized() {
      INSTANCE.ensureInitializedInternalSneaky();
      return null;
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private final static OrPatternQuerySpecification.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    private final PParameter parameter_pEventPattern = new PParameter("eventPattern", "org.eclipse.viatra.cep.core.metamodels.events.ComplexEventPattern", new EClassTransitiveInstancesKey((EClass)getClassifierLiteralSafe("cep.meta", "ComplexEventPattern")), PParameterDirection.INOUT);
    
    private final List<PParameter> parameters = Arrays.asList(parameter_pEventPattern);
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.viatra.cep.core.engine.compiler.orPattern";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("eventPattern");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return parameters;
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() throws QueryInitializationException {
      Set<PBody> bodies = Sets.newLinkedHashSet();
      try {
          {
              PBody body = new PBody(this);
              PVariable var_eventPattern = body.getOrCreateVariableByName("eventPattern");
              PVariable var_operator = body.getOrCreateVariableByName("operator");
              new TypeConstraint(body, Tuples.flatTupleOf(var_eventPattern), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("cep.meta", "ComplexEventPattern")));
              body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
                 new ExportedParameter(body, var_eventPattern, parameter_pEventPattern)
              ));
              // 	find complexPattern(eventPattern, operator)
              new PositivePatternCall(body, new FlatTuple(var_eventPattern, var_operator), ComplexPatternQuerySpecification.instance().getInternalQueryRepresentation());
              // 	OR(operator)
              new TypeConstraint(body, Tuples.flatTupleOf(var_operator), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("cep.meta", "OR")));
              bodies.add(body);
          }
      } catch (ViatraQueryException ex) {
          throw processDependencyException(ex);
      }
      return bodies;
    }
  }
}
