/**
 * Copyright (c) 2010-2017, Grill Balázs, IncQueryLabs
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Grill Balázs - initial API and implementation
 */
package org.eclipse.viatra.query.testing.core.internal;

import java.util.Map;
import org.eclipse.viatra.query.runtime.api.IPatternMatch;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngineOptions;
import org.eclipse.viatra.query.runtime.api.ViatraQueryMatcher;
import org.eclipse.viatra.query.runtime.emf.EMFScope;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.backend.QueryEvaluationHint;
import org.eclipse.viatra.query.testing.core.PatternBasedMatchSetModelProvider;
import org.eclipse.viatra.query.testing.core.api.IPatternExecutionAnalyzer;
import org.eclipse.viatra.query.testing.core.api.JavaObjectAccess;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Functions.Function2;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * This implementation adds support to call analyzers
 * 
 * @since 1.6
 */
@SuppressWarnings("all")
public class AnalyzedPatternBasedMatchSetModelProvider extends PatternBasedMatchSetModelProvider {
  private final Iterable<? extends IPatternExecutionAnalyzer> analyzers;
  
  public AnalyzedPatternBasedMatchSetModelProvider(final QueryEvaluationHint hint, final Map<String, JavaObjectAccess> accessmap, final Iterable<? extends IPatternExecutionAnalyzer> analyzers) {
    super(hint, accessmap);
    this.analyzers = analyzers;
  }
  
  @Override
  protected ViatraQueryEngineOptions getEngineOptions() {
    ViatraQueryEngineOptions _xblockexpression = null;
    {
      final ViatraQueryEngineOptions superOptions = super.getEngineOptions();
      QueryEvaluationHint _engineDefaultHints = superOptions.getEngineDefaultHints();
      final Function2<QueryEvaluationHint, IPatternExecutionAnalyzer, QueryEvaluationHint> _function = new Function2<QueryEvaluationHint, IPatternExecutionAnalyzer, QueryEvaluationHint>() {
        @Override
        public QueryEvaluationHint apply(final QueryEvaluationHint r, final IPatternExecutionAnalyzer t) {
          return t.configure(r);
        }
      };
      final QueryEvaluationHint updatedHint = IterableExtensions.fold(this.analyzers, _engineDefaultHints, _function);
      ViatraQueryEngineOptions.Builder _copyOptions = ViatraQueryEngineOptions.copyOptions(superOptions);
      ViatraQueryEngineOptions.Builder _withDefaultHint = _copyOptions.withDefaultHint(updatedHint);
      _xblockexpression = _withDefaultHint.build();
    }
    return _xblockexpression;
  }
  
  @Override
  protected <Match extends IPatternMatch> ViatraQueryMatcher<Match> createMatcher(final EMFScope scope, final IQuerySpecification<? extends ViatraQueryMatcher<Match>> querySpecification, final QueryEvaluationHint hint) throws ViatraQueryException {
    final ViatraQueryMatcher<Match> matcher = super.<Match>createMatcher(scope, querySpecification, hint);
    final Procedure1<IPatternExecutionAnalyzer> _function = new Procedure1<IPatternExecutionAnalyzer>() {
      @Override
      public void apply(final IPatternExecutionAnalyzer it) {
        try {
          it.processMatcher(matcher);
        } catch (Throwable _e) {
          throw Exceptions.sneakyThrow(_e);
        }
      }
    };
    IterableExtensions.forEach(this.analyzers, _function);
    return matcher;
  }
}
