/**
 * Copyright (c) 2010-2016, Peter Lunk, IncQuery Labs Ltd.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Peter Lunk - initial API and implementation
 */
package org.eclipse.viatra.query.testing.core.internal;

import com.google.common.base.Equivalence;
import com.google.common.base.Objects;
import com.google.common.collect.MapDifference;
import com.google.common.collect.Maps;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.viatra.query.testing.core.api.JavaObjectAccess;
import org.eclipse.viatra.query.testing.core.api.MatchRecordEquivalence;
import org.eclipse.viatra.query.testing.snapshot.MatchRecord;
import org.eclipse.viatra.query.testing.snapshot.MatchSubstitutionRecord;
import org.eclipse.viatra.query.testing.snapshot.SerializedJavaObjectSubstitution;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.Functions.Function2;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;

/**
 * @author Peter Lunk
 */
@SuppressWarnings("all")
public class DefaultMatchRecordEquivalence extends MatchRecordEquivalence {
  public DefaultMatchRecordEquivalence(final Map<String, JavaObjectAccess> accessMap) {
    super(accessMap);
  }
  
  public DefaultMatchRecordEquivalence() {
    super(Maps.<String, JavaObjectAccess>newHashMap());
  }
  
  /**
   * (non-Javadoc)
   * @see Equivalence#doEquivalent(java.lang.Object, java.lang.Object)
   */
  @Override
  protected boolean doEquivalent(final MatchRecord a, final MatchRecord b) {
    HashMap<String, Object> _map = this.toMap(a);
    HashMap<String, Object> _map_1 = this.toMap(b);
    MapDifference<String, Object> _difference = Maps.<String, Object>difference(_map, _map_1, new Equivalence<Object>() {
      @Override
      protected boolean doEquivalent(final Object a, final Object b) {
        boolean _and = false;
        if (!(a instanceof SerializedJavaObjectSubstitution)) {
          _and = false;
        } else {
          _and = (b instanceof SerializedJavaObjectSubstitution);
        }
        if (_and) {
          final SerializedJavaObjectSubstitution aSub = ((SerializedJavaObjectSubstitution) a);
          final SerializedJavaObjectSubstitution bSub = ((SerializedJavaObjectSubstitution) b);
          String _type = aSub.getType();
          String _type_1 = bSub.getType();
          boolean _equals = _type.equals(_type_1);
          if (_equals) {
            String _type_2 = aSub.getType();
            final JavaObjectAccess access = DefaultMatchRecordEquivalence.this.accessMap.get(_type_2);
            return access.equals(aSub, bSub);
          }
        } else {
          boolean _and_1 = false;
          if (!(a instanceof InternalEObject)) {
            _and_1 = false;
          } else {
            _and_1 = (b instanceof InternalEObject);
          }
          if (_and_1) {
            boolean _and_2 = false;
            boolean _eIsProxy = ((InternalEObject) a).eIsProxy();
            if (!_eIsProxy) {
              _and_2 = false;
            } else {
              boolean _eIsProxy_1 = ((InternalEObject) b).eIsProxy();
              _and_2 = _eIsProxy_1;
            }
            if (_and_2) {
              URI _eProxyURI = ((InternalEObject) a).eProxyURI();
              URI _eProxyURI_1 = ((InternalEObject) b).eProxyURI();
              return Objects.equal(_eProxyURI, _eProxyURI_1);
            }
          }
        }
        return java.util.Objects.equals(a, b);
      }
      
      @Override
      protected int doHash(final Object t) {
        int _xblockexpression = (int) 0;
        {
          boolean _and = false;
          if (!(t instanceof SerializedJavaObjectSubstitution)) {
            _and = false;
          } else {
            boolean _eIsProxy = ((InternalEObject) t).eIsProxy();
            boolean _not = (!_eIsProxy);
            _and = _not;
          }
          if (_and) {
            String _type = ((SerializedJavaObjectSubstitution) t).getType();
            final JavaObjectAccess access = DefaultMatchRecordEquivalence.this.accessMap.get(_type);
            return access.calculateHash(((SerializedJavaObjectSubstitution) t));
          } else {
            boolean _and_1 = false;
            if (!(t instanceof InternalEObject)) {
              _and_1 = false;
            } else {
              boolean _eIsProxy_1 = ((InternalEObject) t).eIsProxy();
              _and_1 = _eIsProxy_1;
            }
            if (_and_1) {
              URI _eProxyURI = ((InternalEObject) t).eProxyURI();
              return _eProxyURI.hashCode();
            }
          }
          _xblockexpression = t.hashCode();
        }
        return _xblockexpression;
      }
    });
    return _difference.areEqual();
  }
  
  /**
   * (non-Javadoc)
   * @see Equivalence#doHash(java.lang.Object)
   */
  @Override
  protected int doHash(final MatchRecord t) {
    EList<MatchSubstitutionRecord> _substitutions = t.getSubstitutions();
    final Function1<MatchSubstitutionRecord, Integer> _function = new Function1<MatchSubstitutionRecord, Integer>() {
      @Override
      public Integer apply(final MatchSubstitutionRecord it) {
        int _xblockexpression = (int) 0;
        {
          final Object value = DefaultMatchRecordEquivalence.this.helper.derivedValue(it);
          int _xifexpression = (int) 0;
          boolean _and = false;
          if (!(value instanceof InternalEObject)) {
            _and = false;
          } else {
            boolean _eIsProxy = ((InternalEObject) value).eIsProxy();
            _and = _eIsProxy;
          }
          if (_and) {
            URI _eProxyURI = ((InternalEObject) value).eProxyURI();
            _xifexpression = _eProxyURI.hashCode();
          } else {
            int _xifexpression_1 = (int) 0;
            Object _derivedValue = DefaultMatchRecordEquivalence.this.helper.derivedValue(it);
            if ((_derivedValue instanceof SerializedJavaObjectSubstitution)) {
              Object _derivedValue_1 = DefaultMatchRecordEquivalence.this.helper.derivedValue(it);
              String _type = ((SerializedJavaObjectSubstitution) _derivedValue_1).getType();
              final JavaObjectAccess access = DefaultMatchRecordEquivalence.this.accessMap.get(_type);
              Object _derivedValue_2 = DefaultMatchRecordEquivalence.this.helper.derivedValue(it);
              return Integer.valueOf(access.calculateHash(((SerializedJavaObjectSubstitution) _derivedValue_2)));
            } else {
              Object _derivedValue_3 = DefaultMatchRecordEquivalence.this.helper.derivedValue(it);
              int _hashCode = 0;
              if (_derivedValue_3!=null) {
                _hashCode=_derivedValue_3.hashCode();
              }
              _xifexpression_1 = _hashCode;
            }
            _xifexpression = _xifexpression_1;
          }
          _xblockexpression = _xifexpression;
        }
        return Integer.valueOf(_xblockexpression);
      }
    };
    List<Integer> _map = ListExtensions.<MatchSubstitutionRecord, Integer>map(_substitutions, _function);
    final Function2<Integer, Integer, Integer> _function_1 = new Function2<Integer, Integer, Integer>() {
      @Override
      public Integer apply(final Integer r, final Integer e) {
        return Integer.valueOf(((r).intValue() + (e).intValue()));
      }
    };
    return (int) IterableExtensions.<Integer, Integer>fold(_map, Integer.valueOf(0), _function_1);
  }
}
