/**
 * 
 *   Copyright (c) 2010-2015, Csaba Debreceni, Istvan Rath and Daniel Varro
 *   All rights reserved. This program and the accompanying materials
 *   are made available under the terms of the Eclipse Public License v1.0
 *   which accompanies this distribution, and is available at
 *   http://www.eclipse.org/legal/epl-v10.html
 *  
 *   Contributors:
 *     Csaba Debreceni - initial API and implementation
 *  
 */
package org.eclipse.viatra.transformation.views.traceability.patterns;

import com.google.common.collect.Sets;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;
import org.eclipse.viatra.query.runtime.api.IPatternMatch;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.viatra.query.runtime.api.impl.BaseGeneratedEMFQuerySpecification;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.api.impl.BasePatternMatch;
import org.eclipse.viatra.query.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.viatra.query.runtime.emf.types.EDataTypeInSlotsKey;
import org.eclipse.viatra.query.runtime.emf.types.EStructuralFeatureInstancesKey;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.context.common.JavaTransitiveInstancesKey;
import org.eclipse.viatra.query.runtime.matchers.psystem.PBody;
import org.eclipse.viatra.query.runtime.matchers.psystem.PVariable;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.Equality;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicdeferred.TypeFilterConstraint;
import org.eclipse.viatra.query.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.PParameterDirection;
import org.eclipse.viatra.query.runtime.matchers.psystem.queries.QueryInitializationException;
import org.eclipse.viatra.query.runtime.matchers.tuple.FlatTuple;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuples;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;
import org.eclipse.viatra.transformation.views.traceability.Trace;

/**
 * A pattern-specific query specification that can instantiate Matcher in a type-safe way.
 * 
 * @see Matcher
 * @see Match
 * 
 */
@SuppressWarnings("all")
public final class Trace2object extends BaseGeneratedEMFQuerySpecification<Trace2object.Matcher> {
  /**
   * Pattern-specific match representation of the org.eclipse.viatra.transformation.views.traceability.patterns.trace2object pattern,
   * to be used in conjunction with {@link Matcher}.
   * 
   * <p>Class fields correspond to parameters of the pattern. Fields with value null are considered unassigned.
   * Each instance is a (possibly partial) substitution of pattern parameters,
   * usable to represent a match of the pattern in the result of a query,
   * or to specify the bound (fixed) input parameters when issuing a query.
   * 
   * @see Matcher
   * 
   */
  public static abstract class Match extends BasePatternMatch {
    private Object fParam;
    
    private Trace fTrace;
    
    private String fId;
    
    private static List<String> parameterNames = makeImmutableList("param", "trace", "id");
    
    private Match(final Object pParam, final Trace pTrace, final String pId) {
      this.fParam = pParam;
      this.fTrace = pTrace;
      this.fId = pId;
    }
    
    @Override
    public Object get(final String parameterName) {
      if ("param".equals(parameterName)) return this.fParam;
      if ("trace".equals(parameterName)) return this.fTrace;
      if ("id".equals(parameterName)) return this.fId;
      return null;
    }
    
    public Object getParam() {
      return this.fParam;
    }
    
    public Trace getTrace() {
      return this.fTrace;
    }
    
    public String getId() {
      return this.fId;
    }
    
    @Override
    public boolean set(final String parameterName, final Object newValue) {
      if (!isMutable()) throw new java.lang.UnsupportedOperationException();
      if ("param".equals(parameterName) && newValue instanceof Object) {
          this.fParam = (Object) newValue;
          return true;
      }
      if ("trace".equals(parameterName) ) {
          this.fTrace = (Trace) newValue;
          return true;
      }
      if ("id".equals(parameterName) ) {
          this.fId = (String) newValue;
          return true;
      }
      return false;
    }
    
    public void setParam(final Object pParam) {
      if (!isMutable()) throw new java.lang.UnsupportedOperationException();
      this.fParam = pParam;
    }
    
    public void setTrace(final Trace pTrace) {
      if (!isMutable()) throw new java.lang.UnsupportedOperationException();
      this.fTrace = pTrace;
    }
    
    public void setId(final String pId) {
      if (!isMutable()) throw new java.lang.UnsupportedOperationException();
      this.fId = pId;
    }
    
    @Override
    public String patternName() {
      return "org.eclipse.viatra.transformation.views.traceability.patterns.trace2object";
    }
    
    @Override
    public List<String> parameterNames() {
      return Trace2object.Match.parameterNames;
    }
    
    @Override
    public Object[] toArray() {
      return new Object[]{fParam, fTrace, fId};
    }
    
    @Override
    public Trace2object.Match toImmutable() {
      return isMutable() ? newMatch(fParam, fTrace, fId) : this;
    }
    
    @Override
    public String prettyPrint() {
      StringBuilder result = new StringBuilder();
      result.append("\"param\"=" + prettyPrintValue(fParam) + ", ");
      result.append("\"trace\"=" + prettyPrintValue(fTrace) + ", ");
      result.append("\"id\"=" + prettyPrintValue(fId));
      return result.toString();
    }
    
    @Override
    public int hashCode() {
      return Objects.hash(fParam, fTrace, fId);
    }
    
    @Override
    public boolean equals(final Object obj) {
      if (this == obj)
          return true;
      if (obj == null) {
          return false;
      }
      if ((obj instanceof Trace2object.Match)) {
          Trace2object.Match other = (Trace2object.Match) obj;
          return Objects.equals(fParam, other.fParam) && Objects.equals(fTrace, other.fTrace) && Objects.equals(fId, other.fId);
      } else {
          // this should be infrequent
          if (!(obj instanceof IPatternMatch)) {
              return false;
          }
          IPatternMatch otherSig  = (IPatternMatch) obj;
          return Objects.equals(specification(), otherSig.specification()) && Arrays.deepEquals(toArray(), otherSig.toArray());
      }
    }
    
    @Override
    public Trace2object specification() {
      try {
          return Trace2object.instance();
      } catch (ViatraQueryException ex) {
           // This cannot happen, as the match object can only be instantiated if the query specification exists
           throw new IllegalStateException (ex);
      }
    }
    
    /**
     * Returns an empty, mutable match.
     * Fields of the mutable match can be filled to create a partial match, usable as matcher input.
     * 
     * @return the empty match.
     * 
     */
    public static Trace2object.Match newEmptyMatch() {
      return new Mutable(null, null, null);
    }
    
    /**
     * Returns a mutable (partial) match.
     * Fields of the mutable match can be filled to create a partial match, usable as matcher input.
     * 
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pId the fixed value of pattern parameter id, or null if not bound.
     * @return the new, mutable (partial) match object.
     * 
     */
    public static Trace2object.Match newMutableMatch(final Object pParam, final Trace pTrace, final String pId) {
      return new Mutable(pParam, pTrace, pId);
    }
    
    /**
     * Returns a new (partial) match.
     * This can be used e.g. to call the matcher with a partial match.
     * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pId the fixed value of pattern parameter id, or null if not bound.
     * @return the (partial) match object.
     * 
     */
    public static Trace2object.Match newMatch(final Object pParam, final Trace pTrace, final String pId) {
      return new Immutable(pParam, pTrace, pId);
    }
    
    private static final class Mutable extends Trace2object.Match {
      Mutable(final Object pParam, final Trace pTrace, final String pId) {
        super(pParam, pTrace, pId);
      }
      
      @Override
      public boolean isMutable() {
        return true;
      }
    }
    
    private static final class Immutable extends Trace2object.Match {
      Immutable(final Object pParam, final Trace pTrace, final String pId) {
        super(pParam, pTrace, pId);
      }
      
      @Override
      public boolean isMutable() {
        return false;
      }
    }
  }
  
  /**
   * Generated pattern matcher API of the org.eclipse.viatra.transformation.views.traceability.patterns.trace2object pattern,
   * providing pattern-specific query methods.
   * 
   * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
   * e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}.
   * 
   * <p>Matches of the pattern will be represented as {@link Match}.
   * 
   * <p>Original source:
   * <code><pre>
   * pattern trace2object(param : java Object, trace : Trace, id : java String) {
   * 	Trace.objects(trace, param);
   * 	Trace.id(trace, id);
   * }
   * </pre></code>
   * 
   * @see Match
   * @see Trace2object
   * 
   */
  public static class Matcher extends BaseMatcher<Trace2object.Match> {
    /**
     * Initializes the pattern matcher within an existing VIATRA Query engine.
     * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
     * The match set will be incrementally refreshed upon updates.
     * @param engine the existing VIATRA Query engine in which this matcher will be created.
     * @throws ViatraQueryException if an error occurs during pattern matcher creation
     * 
     */
    public static Trace2object.Matcher on(final ViatraQueryEngine engine) throws ViatraQueryException {
      // check if matcher already exists
      Matcher matcher = engine.getExistingMatcher(querySpecification());
      if (matcher == null) {
          matcher = (Matcher)engine.getMatcher(querySpecification());
      }
      return matcher;
    }
    
    /**
     * @throws ViatraQueryException if an error occurs during pattern matcher creation
     * @return an initialized matcher
     * @noreference This method is for internal matcher initialization by the framework, do not call it manually.
     * 
     */
    public static Trace2object.Matcher create() throws ViatraQueryException {
      return new Matcher();
    }
    
    private final static int POSITION_PARAM = 0;
    
    private final static int POSITION_TRACE = 1;
    
    private final static int POSITION_ID = 2;
    
    private final static Logger LOGGER = ViatraQueryLoggingUtil.getLogger(Trace2object.Matcher.class);
    
    /**
     * Initializes the pattern matcher within an existing VIATRA Query engine.
     * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
     * The match set will be incrementally refreshed upon updates.
     * @param engine the existing VIATRA Query engine in which this matcher will be created.
     * @throws ViatraQueryException if an error occurs during pattern matcher creation
     * 
     */
    private Matcher() throws ViatraQueryException {
      super(querySpecification());
    }
    
    /**
     * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pId the fixed value of pattern parameter id, or null if not bound.
     * @return matches represented as a Match object.
     * 
     */
    public Collection<Trace2object.Match> getAllMatches(final Object pParam, final Trace pTrace, final String pId) {
      return rawGetAllMatches(new Object[]{pParam, pTrace, pId});
    }
    
    /**
     * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
     * Neither determinism nor randomness of selection is guaranteed.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pId the fixed value of pattern parameter id, or null if not bound.
     * @return a match represented as a Match object, or null if no match is found.
     * 
     */
    public Trace2object.Match getOneArbitraryMatch(final Object pParam, final Trace pTrace, final String pId) {
      return rawGetOneArbitraryMatch(new Object[]{pParam, pTrace, pId});
    }
    
    /**
     * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
     * under any possible substitution of the unspecified parameters (if any).
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pId the fixed value of pattern parameter id, or null if not bound.
     * @return true if the input is a valid (partial) match of the pattern.
     * 
     */
    public boolean hasMatch(final Object pParam, final Trace pTrace, final String pId) {
      return rawHasMatch(new Object[]{pParam, pTrace, pId});
    }
    
    /**
     * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pId the fixed value of pattern parameter id, or null if not bound.
     * @return the number of pattern matches found.
     * 
     */
    public int countMatches(final Object pParam, final Trace pTrace, final String pId) {
      return rawCountMatches(new Object[]{pParam, pTrace, pId});
    }
    
    /**
     * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
     * Neither determinism nor randomness of selection is guaranteed.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pId the fixed value of pattern parameter id, or null if not bound.
     * @param processor the action that will process the selected match.
     * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
     * 
     */
    public boolean forOneArbitraryMatch(final Object pParam, final Trace pTrace, final String pId, final IMatchProcessor<? super Trace2object.Match> processor) {
      return rawForOneArbitraryMatch(new Object[]{pParam, pTrace, pId}, processor);
    }
    
    /**
     * Returns a new (partial) match.
     * This can be used e.g. to call the matcher with a partial match.
     * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
     * @param pParam the fixed value of pattern parameter param, or null if not bound.
     * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
     * @param pId the fixed value of pattern parameter id, or null if not bound.
     * @return the (partial) match object.
     * 
     */
    public Trace2object.Match newMatch(final Object pParam, final Trace pTrace, final String pId) {
      return Trace2object.Match.newMatch(pParam, pTrace, pId);
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    protected Set<Object> rawAccumulateAllValuesOfparam(final Object[] parameters) {
      Set<Object> results = new HashSet<Object>();
      rawAccumulateAllValues(POSITION_PARAM, parameters, results);
      return results;
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Object> getAllValuesOfparam() {
      return rawAccumulateAllValuesOfparam(emptyArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Object> getAllValuesOfparam(final Trace2object.Match partialMatch) {
      return rawAccumulateAllValuesOfparam(partialMatch.toArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for param.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Object> getAllValuesOfparam(final Trace pTrace, final String pId) {
      return rawAccumulateAllValuesOfparam(new Object[]{
      null, 
      pTrace, 
      pId
      });
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    protected Set<Trace> rawAccumulateAllValuesOftrace(final Object[] parameters) {
      Set<Trace> results = new HashSet<Trace>();
      rawAccumulateAllValues(POSITION_TRACE, parameters, results);
      return results;
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Trace> getAllValuesOftrace() {
      return rawAccumulateAllValuesOftrace(emptyArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Trace> getAllValuesOftrace(final Trace2object.Match partialMatch) {
      return rawAccumulateAllValuesOftrace(partialMatch.toArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for trace.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<Trace> getAllValuesOftrace(final Object pParam, final String pId) {
      return rawAccumulateAllValuesOftrace(new Object[]{
      pParam, 
      null, 
      pId
      });
    }
    
    /**
     * Retrieve the set of values that occur in matches for id.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    protected Set<String> rawAccumulateAllValuesOfid(final Object[] parameters) {
      Set<String> results = new HashSet<String>();
      rawAccumulateAllValues(POSITION_ID, parameters, results);
      return results;
    }
    
    /**
     * Retrieve the set of values that occur in matches for id.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<String> getAllValuesOfid() {
      return rawAccumulateAllValuesOfid(emptyArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for id.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<String> getAllValuesOfid(final Trace2object.Match partialMatch) {
      return rawAccumulateAllValuesOfid(partialMatch.toArray());
    }
    
    /**
     * Retrieve the set of values that occur in matches for id.
     * @return the Set of all values or empty set if there are no matches
     * 
     */
    public Set<String> getAllValuesOfid(final Object pParam, final Trace pTrace) {
      return rawAccumulateAllValuesOfid(new Object[]{
      pParam, 
      pTrace, 
      null
      });
    }
    
    @Override
    protected Trace2object.Match tupleToMatch(final Tuple t) {
      try {
          return Trace2object.Match.newMatch((Object) t.get(POSITION_PARAM), (Trace) t.get(POSITION_TRACE), (String) t.get(POSITION_ID));
      } catch(ClassCastException e) {
          LOGGER.error("Element(s) in tuple not properly typed!",e);
          return null;
      }
    }
    
    @Override
    protected Trace2object.Match arrayToMatch(final Object[] match) {
      try {
          return Trace2object.Match.newMatch((Object) match[POSITION_PARAM], (Trace) match[POSITION_TRACE], (String) match[POSITION_ID]);
      } catch(ClassCastException e) {
          LOGGER.error("Element(s) in array not properly typed!",e);
          return null;
      }
    }
    
    @Override
    protected Trace2object.Match arrayToMatchMutable(final Object[] match) {
      try {
          return Trace2object.Match.newMutableMatch((Object) match[POSITION_PARAM], (Trace) match[POSITION_TRACE], (String) match[POSITION_ID]);
      } catch(ClassCastException e) {
          LOGGER.error("Element(s) in array not properly typed!",e);
          return null;
      }
    }
    
    /**
     * @return the singleton instance of the query specification of this pattern
     * @throws ViatraQueryException if the pattern definition could not be loaded
     * 
     */
    public static IQuerySpecification<Trace2object.Matcher> querySpecification() throws ViatraQueryException {
      return Trace2object.instance();
    }
  }
  
  private Trace2object() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static Trace2object instance() throws ViatraQueryException {
    try{
        return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
        throw processInitializerError(err);
    }
  }
  
  @Override
  protected Trace2object.Matcher instantiate(final ViatraQueryEngine engine) throws ViatraQueryException {
    return Trace2object.Matcher.on(engine);
  }
  
  @Override
  public Trace2object.Matcher instantiate() throws ViatraQueryException {
    return Trace2object.Matcher.create();
  }
  
  @Override
  public Trace2object.Match newEmptyMatch() {
    return Trace2object.Match.newEmptyMatch();
  }
  
  @Override
  public Trace2object.Match newMatch(final Object... parameters) {
    return Trace2object.Match.newMatch((java.lang.Object) parameters[0], (org.eclipse.viatra.transformation.views.traceability.Trace) parameters[1], (java.lang.String) parameters[2]);
  }
  
  /**
   * Inner class allowing the singleton instance of {@link JvmGenericType: org.eclipse.viatra.transformation.views.traceability.patterns.Trace2object (visibility: PUBLIC, simpleName: Trace2object, identifier: org.eclipse.viatra.transformation.views.traceability.patterns.Trace2object, deprecated: <unset>) (abstract: false, static: false, final: true, packageName: org.eclipse.viatra.transformation.views.traceability.patterns) (interface: false, strictFloatingPoint: false, anonymous: false)} to be created 
   *     <b>not</b> at the class load time of the outer class, 
   *     but rather at the first call to {@link JvmGenericType: org.eclipse.viatra.transformation.views.traceability.patterns.Trace2object (visibility: PUBLIC, simpleName: Trace2object, identifier: org.eclipse.viatra.transformation.views.traceability.patterns.Trace2object, deprecated: <unset>) (abstract: false, static: false, final: true, packageName: org.eclipse.viatra.transformation.views.traceability.patterns) (interface: false, strictFloatingPoint: false, anonymous: false)#instance()}.
   * 
   * <p> This workaround is required e.g. to support recursion.
   * 
   */
  private static class LazyHolder {
    private final static Trace2object INSTANCE = new Trace2object();
    
    /**
     * Statically initializes the query specification <b>after</b> the field {@link #INSTANCE} is assigned.
     * This initialization order is required to support indirect recursion.
     * 
     * <p> The static initializer is defined using a helper field to work around limitations of the code generator.
     * 
     */
    private final static Object STATIC_INITIALIZER = ensureInitialized();
    
    public static Object ensureInitialized() {
      INSTANCE.ensureInitializedInternalSneaky();
      return null;
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private final static Trace2object.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    private final PParameter parameter_pParam = new PParameter("param", "java.lang.Object", new JavaTransitiveInstancesKey(java.lang.Object.class), PParameterDirection.INOUT);
    
    private final PParameter parameter_pTrace = new PParameter("trace", "org.eclipse.viatra.transformation.views.traceability.Trace", new EClassTransitiveInstancesKey((EClass)getClassifierLiteralSafe("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")), PParameterDirection.INOUT);
    
    private final PParameter parameter_pId = new PParameter("id", "java.lang.String", new JavaTransitiveInstancesKey(java.lang.String.class), PParameterDirection.INOUT);
    
    private final List<PParameter> parameters = Arrays.asList(parameter_pParam, parameter_pTrace, parameter_pId);
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.viatra.transformation.views.traceability.patterns.trace2object";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("param","trace","id");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return parameters;
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() throws QueryInitializationException {
      Set<PBody> bodies = Sets.newLinkedHashSet();
      {
          PBody body = new PBody(this);
          PVariable var_param = body.getOrCreateVariableByName("param");
          PVariable var_trace = body.getOrCreateVariableByName("trace");
          PVariable var_id = body.getOrCreateVariableByName("id");
          new TypeFilterConstraint(body, new FlatTuple(var_param), new JavaTransitiveInstancesKey(java.lang.Object.class));
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")));
          new TypeFilterConstraint(body, new FlatTuple(var_id), new JavaTransitiveInstancesKey(java.lang.String.class));
          body.setSymbolicParameters(Arrays.<ExportedParameter>asList(
             new ExportedParameter(body, var_param, parameter_pParam),
             new ExportedParameter(body, var_trace, parameter_pTrace),
             new ExportedParameter(body, var_id, parameter_pId)
          ));
          // 	Trace.objects(trace, param)
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")));
          PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace, var__virtual_0_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace", "objects")));
          new TypeConstraint(body, Tuples.flatTupleOf(var__virtual_0_), new EDataTypeInSlotsKey((EDataType)getClassifierLiteral("http://www.eclipse.org/emf/2002/Ecore", "EJavaObject")));
          new Equality(body, var__virtual_0_, var_param);
          // 	Trace.id(trace, id)
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace")));
          PVariable var__virtual_1_ = body.getOrCreateVariableByName(".virtual{1}");
          new TypeConstraint(body, Tuples.flatTupleOf(var_trace, var__virtual_1_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/viatra/transformation/views/traceability/1.0", "Trace", "id")));
          new TypeConstraint(body, Tuples.flatTupleOf(var__virtual_1_), new EDataTypeInSlotsKey((EDataType)getClassifierLiteral("http://www.eclipse.org/emf/2002/Ecore", "EString")));
          new Equality(body, var__virtual_1_, var_id);
          bodies.add(body);
      }
      return bodies;
    }
  }
}
